import { readable, writable } from 'svelte/store';

function createSettingsStore() {
  const { subscribe, set, update } = writable({});

  return {
    subscribe,
    update: (res_settings, cb) => {
      const recvTime = (new Date()).getTime();

      if (res_settings.status === 200) {
        res_settings.json().then((settings) => {
          if (settings.start)
            settings.start = new Date(settings.start);
          if (settings.end)
            settings.end = new Date(settings.end);
          if (settings.generation)
            settings.generation = new Date(settings.generation);
          if (settings.activateTime)
            settings.activateTime = new Date(settings.activateTime);
          if (!settings.disablesubmitbutton)
            settings.disablesubmitbutton = null;

          settings.recvTime = recvTime;
          const x_fic_time = res_settings.headers.get("x-fic-time");
          if (x_fic_time) {
            settings.currentTime = Math.floor(x_fic_time * 1000);
          } else {
            settings.currentTime = settings.recvTime;
          }

          update((s) => (Object.assign({}, settings)));

          if (cb) {
            cb(settings);
          }
        });
      }
    },
  }
}

export const settings = createSettingsStore();

function updateTime(settings) {
  const time = {};

  const srv_cur = new Date(Date.now() + (settings.currentTime - settings.recvTime));

  let remain = 0;
  if (settings.start === undefined || settings.start == 0) {
    return time;
  } else if (settings.start > srv_cur) {
    time.startIn = Math.floor((settings.start - srv_cur) / 1000);
    remain = settings.end - settings.start;
  } else if (settings.end > srv_cur) {
    time.startIn = 0;
    remain = settings.end - srv_cur;
  }

  time.progression = 1 - remain / (settings.end - settings.start);

  remain = remain / 1000;

  if (remain < 0) {
    remain = 0;
    time.end = true;
    time.expired = true;
  } else if (remain < 60) {
    time.end = false;
    time.expired = true;
  } else {
    time.end = false;
    time.expired = false;
  }

  time.remaining = remain;
  time.hours = Math.floor(remain / 3600);
  time.minutes = Math.floor((remain % 3600) / 60);
  time.seconds = Math.floor(remain % 60);

  if (time.hours <= 9) {
    time.hours = "0" + time.hours;
  }
  if (time.minutes <= 9) {
    time.minutes = "0" + time.minutes;
  }
  if (time.seconds <= 9) {
    time.seconds = "0" + time.seconds;
  }

  return time;
}

export const time = readable({}, function start(set) {
  let _settings = {};

  const unsubscribe = settings.subscribe((settings) => {
    _settings = settings;
  });

  const interval = setInterval(() => {
    set(updateTime(_settings));
  }, 1000);

  return function stop() {
    clearInterval(interval);
    unsubscribe();
  }
});
