import { derived, writable } from 'svelte/store';

function createThemesStore() {
  const { subscribe, set, update } = writable({themes: null, exercices_idx: {}, max_solved: 0});

  return {
    subscribe,
    update: async (res_themes, cb=null) => {
      if (res_themes.status === 200) {
        const themes = await res_themes.json();

        let max_solved = 0;
        const exercices_idx = {};

        for (let key in themes) {
          const theme = themes[key];

          if (theme.solved > max_solved) {
            max_solved = theme.solved;
          }

	  themes[key].exercice_count = Object.keys(theme.exercices).length;
	  themes[key].exercice_coeff_max = 0;
	  themes[key].max_gain = 0;
	  let last_exercice = null;
	  for (let k in theme.exercices) {
            const exercice = theme.exercices[k];

            themes[key].max_gain += exercice.gain;
            if (themes[key].exercice_coeff_max < exercice.curcoeff) {
              themes[key].exercice_coeff_max = exercice.curcoeff;
            }

	    if (last_exercice != null)
	      themes[key].exercices[last_exercice].next = k;
	    last_exercice = k;

            exercice.id = k;
            exercices_idx[k] = exercice;
	  }
        }

        update((t) => (Object.assign(t, {themes, exercices_idx, max_solved})));

        if (cb) {
          cb(themes, exercices_idx, max_solved);
        }
      }
    },
  };
}

export const themesStore = createThemesStore();

export const themes = derived(
  themesStore,
  ($themesStore) => ($themesStore.themes),
);

export const exercices_idx = derived(
  themesStore,
  ($themesStore) => ($themesStore.exercices_idx),
);

export const max_solved = derived(
  themesStore,
  ($themesStore) => ($themesStore.max_solved),
);
