package api

import (
	"strconv"

	"srs.epita.fr/fic-server/libfic"

	"github.com/julienschmidt/httprouter"
)

func init() {
	router.GET("/api/themes", apiHandler(listThemes))
	router.GET("/api/themes.json", apiHandler(exportThemes))

	router.GET("/api/themes/:thid", apiHandler(themeHandler(showTheme)))

	router.GET("/api/themes/:thid/exercices", apiHandler(themeHandler(listThemedExercices)))

	router.GET("/api/themes/:thid/exercices/:eid", apiHandler(exerciceHandler(showExercice)))
}

func themeHandler(f func(QAUser, *fic.Theme, []byte) (interface{}, error)) func(QAUser, httprouter.Params, []byte) (interface{}, error) {
	return func(u QAUser, ps httprouter.Params, body []byte) (interface{}, error) {
		if thid, err := strconv.ParseInt(string(ps.ByName("thid")), 10, 64); err != nil {
			return nil, err
		} else if theme, err := fic.GetTheme(thid); err != nil {
			return nil, err
		} else {
			return f(u, theme, body)
		}
	}
}

func getExercice(args []string) (*fic.Exercice, error) {
	if tid, err := strconv.ParseInt(string(args[0]), 10, 64); err != nil {
		return nil, err
	} else if theme, err := fic.GetTheme(tid); err != nil {
		return nil, err
	} else if eid, err := strconv.Atoi(string(args[1])); err != nil {
		return nil, err
	} else {
		return theme.GetExercice(eid)
	}
}

func listThemes(_ QAUser, _ httprouter.Params, _ []byte) (interface{}, error) {
	return fic.GetThemes()
}

func exportThemes(_ QAUser, _ httprouter.Params, _ []byte) (interface{}, error) {
	return fic.ExportThemes()
}

func showTheme(_ QAUser, theme *fic.Theme, _ []byte) (interface{}, error) {
	return theme, nil
}

func listThemedExercices(_ QAUser, theme *fic.Theme, _ []byte) (interface{}, error) {
	return theme.GetExercices()
}

func showThemedExercice(_ QAUser, theme *fic.Theme, exercice *fic.Exercice, body []byte) (interface{}, error) {
	return exercice, nil
}
